/**
 Functions for the ARM7 to process the commands from the ARM9. Based
 on code from the MOD player example posted to the GBADEV forums.
 */
#include <nds.h>
#include "command.h"
#include "transfer.h"

static void CommandFIFOInit() {
    REG_IPC_FIFO_CR = IPC_FIFO_ENABLE | IPC_FIFO_SEND_CLEAR;
}

static void CommandFIFOSend() {
    static int count = 0;
    REG_IPC_FIFO_TX = ++count;
}

static void CommandFIFORecv() {
    arm7_fifo->recv = REG_IPC_FIFO_RX;
}

static void CommandPlaySound(SoundData * soundData)
{
    s8 channel = soundData->chan;
    
    if (channel < 0) {
        //channel = getFreeSoundChannel();
        for (s8 i = 0; i < 16; i++) {
            if ((SCHANNEL_CR(i) & SCHANNEL_ENABLE) == 0) {
                channel =  i;
                break;
            }
        }
    }
    
    if ((channel >= 0) && (channel < 16)) {
        /*
         sound->data;
         sound->len;
         sound->channel = _sound->channel;
         sound->rate = _sound->rate;
         sound->vol = _sound->vol;
         sound->pan = _sound->pan;
         sound->format = _sound->format;
         sound->loop = sound->loop;
         sound->PADDING = _sound->PADDING;
         */
        
        //SCHANNEL_CR(channel) = 0; 
        SCHANNEL_TIMER(channel)  = SOUND_FREQ(soundData->rate);
        SCHANNEL_SOURCE(channel) = (u32)soundData->data;
        SCHANNEL_LENGTH(channel) = soundData->len >> 2 ;
        //SCHANNEL_PAN(channel) = 100;
        SCHANNEL_CR(channel)     = SCHANNEL_ENABLE | (soundData->loop==true?SOUND_REPEAT:SOUND_ONE_SHOT) | SOUND_VOL(soundData->vol) | SOUND_PAN(soundData->pan) | (soundData->format==1?SOUND_FORMAT_8BIT:SOUND_FORMAT_16BIT);
        
    }
}

void CommandProcessCommands() {
    static int currentCommand = -1;
    
    while(currentCommand != commandControl->currentCommand) {
        Command* command = &commandControl->command[currentCommand];
        switch(command->commandType) {
            case FIFO_INIT:
                CommandFIFOInit();
                break;      
            case FIFO_SEND:
                CommandFIFOSend();
                break;      
            case FIFO_RECV:
                CommandFIFORecv();
                break;
            case PLAY_SOUND:
                CommandPlaySound((SoundData *)&command->data);
                break; 
            default:
                break;
        }
        
        currentCommand++;
        currentCommand &= MAX_COMMANDS-1;
    }
}

