/**
 *  Draggable.h
 *  
 *
 *  Created by Jaeder Patater on 11/25/05.
 *  Copyright 2005. All rights reserved.
 *
 */

#ifndef DRAGGABLE_H
#define DRAGGABLE_H

#include "Button.h"

class Draggable : public Button {
protected:
    uint16 gripX;
    uint16 gripY;
    uint16 prevX;
    uint16 prevY;
    
    void recordGrip(TouchEvent * event);
    
    void recordLoc();
    
    /**
     *  Draggable(char * name, uint16 x, uint16 y, uint16 width, uint16 height)
     *
     *  Creates a Draggable based on the specified name, coordinate, and size
     *
     */
    Draggable(char * name, uint16 x, uint16 y, uint16 width, uint16 height);
    
    Draggable();                            //default constructor
    Draggable(const Draggable & other);     //copy constructor
    
public:
    
    virtual ~Draggable();                   //deconstructor
    Draggable operator=(const Draggable & other); //= operator
    
    /**
     *  recieveEvent(UIEvent * event)
     *
     *  Handles the reception of events passed to this Button. It uses the state of a TouchEvent object to provide press and release functionality.
     *
     */
    virtual void recieveEvent(UIEvent * event);
    
    /**
     *  handleTouchUp()
     *
     *  Loads the map editor via tileEditDS
     *
     */
    virtual void handleTouchUp();
    
    /**
     *  handleTouchDown()
     *
     *  Put the code you want to execute on button press into this method
     *
     */
    virtual void handleTouchDown();
    
    /**
     *  handleTouchHeld()
     *
     *  Put the code you want to execute on button hold into this method
     *
     */
    virtual void handleTouchHeld();
    
    /**
     *  paintPressed()
     *
     *  Paints the object in the pressed state.
     *
     */
    virtual void paintPressed();
    
    /**
     *  paintReleased()
     *
     *  Paints the object in the released state.
     *
     */
    virtual void paintReleased();
    
    /**
     *  paintPressedFocused()
     *
     *  Paints the object in the focused and pressed state.
     *
     *  Put your paint code here (only necessary if you want the object to show focus). Default (paintPressed())
     *
     */
    virtual void paintPressedFocused();
    
    /**
     *  paintReleasedFocused()
     *
     *  Paints the object in the focused and released state.
     *
     *  Put your paint code here (only necessary if you want the object to show focus).
     *
     */
    virtual void paintReleasedFocused();
    
};


#endif

