/**
 *  Guikit.h
 *  
 *  A singleton which includes all the proper guikit includes and contains some helpful functions.
 *
 *  Created by Jaeder Patater on 9/11/05.
 *  Copyright 2005. All rights reserved.
 *
 */

#ifndef GUI_KIT_H
#define GUI_KIT_H

#include <nds.h>
#include <string>

//gui kit includes
#include "Screen.h"
#include "HotSpot.h"
#include "TouchEvent.h"
#include "KeyEvent.h"

//macro returns a boolean expression for two values equal within a given tolerance
//#define FUZZY_EQUALS(v1, v2, tol) ((v1 >= v2) && (v1 <= v2 + tol))

//macro rounds a negative or positive floating point variable to integer type
//#define round(x) ((x)>=0?(long)((x)+0.5):(long)((x)-0.5))

//defined in Screen class to avoid declaring globals more than once (linker errors)

class Guikit {
public:
    static int temp;
    static uint16 touch_x, touch_y;
    static uint16 previous_touch_x, previous_touch_y;
    
    /**
     *  dmaFill32(const u32 fill, void * dest, u32 size)
     *
     *  Uses dma to fill memory with 'fill' value
     *
     */
    static void dmaFill32(const u32 fill, void * dest, u32 size);
    
    /**
     *  vblankSleep(int miliseconds)
     *
     *  Uses vblank to sleep for the specified number of vblanks
     *
     */
    static void vblankSleep(int vblanks);
    
    /**
     *  rtcSleep(int seconds)
     *
     *  Uses the RTC to sleep for the specified number of seconds
     *
     */
    static void rtcSleep(int seconds);
    
    /**
     *  calcFPS(double & fps, double & counter, double & seconds)
     *
     *  Pass it some doubles by reference and it'll set the fps double to the fps every second (uses the RTC).
     *  fps is set to -1 if the seconds have not yet changed.
     *
     */
    static void calcFPS(double & fps, double & counter, double & seconds);
    
    /**
     *  exitToLoader()
     *
     *  Uses the GBAMP to load and execute "/_BOOT_MP.nds".
     *
     */
    static void exitToLoader();
    
    /**
     *  bootNDS(std::string filename)
     *
     *  Uses the GBAMP to load and execute NDS represented by filename.
     *
     */
    static void bootNDS(std::string filename);
    
    /**
     *  updateInput()
     *
     *  Loads touch values for use with the guikit and calls scanKeys().
     *
     */
    static void updateInput();
    
    /**
     *  waitForTouch()
     *
     *  Suspends operations until the screen is touched. Useful for title screens.
     *
     */
    static void waitForTouch();
    
    /**
     *  handleInput()
     *
     *  Translates user input into guikit events.
     *
     */
    static UIEvent * handleInput();
};

#endif

