/**
 *  HotSpot.h
 *  
 *  A touchable rectangle. Can recieve focus by default, overriding UIElement default.
 *
 *  Created by Jaeder Patater on 8/5/05.
 *  Copyright 2005. All rights reserved.
 *
 */

#ifndef HOT_SPOT_H
#define HOT_SPOT_H

#include "UIElement.h"
#include "TouchEvent.h"

class HotSpot : public UIElement {
private:
    
    /**
     *  init(const HotSpot & other)
     *
     *  Initializes this to other
     *
     */
    void init(const HotSpot & other);
    
protected:
    
    uint16 x;               //the x coord of the upper-left corner
    uint16 y;               //the y coord of the upper-left corner
    uint16 width;           //the width of the hot spot
    uint16 height;          //the height of the hot spot
    
    //constructors are now protected, because with this new, pure model (7 November 2005 and beyond), you
    //are expected to create a class for each hotspot-handler combo (for each handler, you create a hotspot class)
    // i.e. NewGameHotSpot, KickHotSpot, SaveHotSpot, etc.
    
    /**
     *  HotSpot(unsigned int x, unsigned int y, unsigned int width, unsigned int height)
     *
     *  Creates a hotspot based on the specified coordinate, and size
     *
     */
    HotSpot(uint16 x, uint16 y, uint16 width, uint16 height);
    
    HotSpot();                              //default constructor
    HotSpot(const HotSpot & other);         //copy constructor
    
public:
    
    virtual ~HotSpot();                     //deconstructor

    HotSpot operator=(const HotSpot & other);   //= operator
    
    uint16 getX() const;
    
    uint16 getY() const;
    
    uint16 getWidth() const;
    
    uint16 getHeight() const;
    
    void setX(uint16 x);
    
    void setY(uint16 y);
    
    void setWidth(uint16 width);
    
    void setHeight(uint16 height);
    
    /**
     *  handleTouch()
     *
     *  The function to be called when handling a touch event.
     *  When inheriting from this class, simply override this method with what you want your hotspot to do.
     *
     */
    virtual void handleTouch(TouchEvent * touchEvent);
    
    /**
     *  recieveEvent(UIEvent * event)
     *
     *  Handle the recieving and executing of recieved events.
     *
     */
    virtual void recieveEvent(UIEvent * event);
    
};

#endif

