
#ifndef CS240_LINKED_LIST_H
#define CS240_LINKED_LIST_H


#include <iostream>


/**
LLNode implements a doubly-linked list node
 */
template <class T> class LinkedList;

template <class T>
class LLNode {
    friend class LinkedList<T>; // LinkedList can access private members of LLNode
    
private:
    const T value;              // value stored in the node
    LLNode<T> * prev;               // pointer to previous node in the list
    LLNode<T> * next;               // pointer to next node in the list
    
public:
    
    LLNode(const T v, LLNode<T> * p, LLNode<T> * n):
        value(v), prev(p), next(n) {
        return;
    }
    
    const T GetValue() {
        return value;
    }
    
    LLNode<T> * GetPrevious() {
        return prev;
    }
    
    LLNode<T> * GetNext() {
        return next;
    }
    
};


/**
LinkedList implements a doubly-linked list
 */
template <class T>
class LinkedList {
private:
    LLNode<T> * head;
    LLNode<T> * tail;
    int size;
    int count;
    
    LLNode<T> * FindHelp(const T v, LLNode<T> * n) const {
        if (n == NULL) {
            return NULL;
        }
        
        if (v == n->value) {
            return n;
        } else {
            return (FindHelp(v, n->next));
        }
    }
    
    void Init() {
        head = NULL;
        tail = NULL;
        size = 0;
    }
    
    void Init(const LinkedList * other) {
        if (other == NULL) {
            //or should I go: 
            //this = NULL or something...
            Free();
            Init();
        } else {
            Free();
            Init();
            Init(other->head, head);
        }
    }
    
    void Init(const LLNode<T> * other, LLNode<T> * n) {
        if (other == NULL) {
            tail = n;
            return;
        }
        
        n = Insert(other->value, n);
        Init(other->next, n);
        return;
    }
    
    void Free() {
        Free(head);
        Init();
    }
    
    void Free(LLNode<T> * node) {
        if (node == NULL) {
            return;
        }
        
        Free(node->next);
        delete node;
    }
    
public:
    
    LinkedList() {
        Init();
        count = 0;
    }
    
    LinkedList(const LinkedList & other) {
        Init(&other);
    }
    
    ~LinkedList() {
        Free();
    }
    
    void operator =(const LinkedList & other) {
        Init(&other);
    }
    
    bool IsEmpty() const {
        return (size == 0);
    }
    
    void Clear() {
        Free();
    }
    
    int GetSize() const {
        return size;
    }
    
    LLNode<T> * GetFirst() const {
        return head;
    }
    
    LLNode<T> * GetLast() const {
        return tail;
    }
    
    LLNode<T> * Insert(const T v, LLNode<T> * n) {
        LLNode<T> * temp = new LLNode<T>(v, 0, 0);
        
        
        if (n == NULL) {
            //cout << "n is NULL:" << ++count << endl;
            if (head != NULL) {
                //cout << "head is not NULL:" << head << endl;
                temp->next = head;
            } else {
                //cout << "head is NULL:" << head << endl;
                temp->next = NULL;
                tail = temp;
            }
            head = temp;
            temp->prev = NULL;
            if (temp->next != NULL) {
                temp->next->prev = temp;
            }
            size++;
            return temp;
        } else {
            //cout << "n is not NULL" << endl;
            temp->next = n->next;
            if (n->next != NULL) {
                n->next->prev = temp;
            } else {
                tail = temp;
            }
            n->next = temp;
            temp->prev = n;
            size++;
            return temp;
        }
    }
    
    LLNode<T> * Find(const T v, LLNode<T> * n) const {
        if (n == NULL) {
            return FindHelp(v, head);
        } else {
            return FindHelp(v, n->next);
        }
    }
    
    void Remove(LLNode<T> * n) {
        if (n != NULL) {
            //cout << "remove" << endl;
            //cout << "remove:" << n->GetValue()->getURL()->getAbsoluteURL() << endl;
            if (n == head) {
                //cout << "head" << endl;
                head = n->next;
                if (head != NULL) {
                    head->prev = NULL;
                }
            } else if (n == tail) {
                //cout << "tail" << endl;
                tail = n->prev;
                if (tail != NULL) {
                    tail->next = NULL;
                }
            } else {
                //cout << "nop" << endl;
                n->prev->next = n->next;
                n->next->prev = n->prev;
            }
            
            size--;
            delete n;
        }
        if (size == 0) {
            tail = NULL;
        }
    }
    
};

#endif

