/**
 *  MenuScreen.h
 *  
 *  A Screen (specifically SplashScreen) that contains a list of buttons for use as a menu. A great class to use for a title screen.
 *
 *  All these methods can easily be done with a regular Screen or SplashScreen, but this class simply provides a nice structure for a programmer to use when creating full screen menus.
 *
 *  Created by Jaeder Patater on 11/7/05.
 *  Copyright 2005. All rights reserved.
 *
 */

#ifndef MENU_SCREEN_H
#define MENU_SCREEN_H

#include "SplashScreen.h"
#include "Button.h"


class MenuScreen : public SplashScreen {
protected:
    
    //Constructors defined as protected so that the programmer -knows to/must- use this class as abstract.
    
    /**
     *  MenuScreen(ScreenType _screenType, ScreenMode _screenMode)
     *
     *  Creates a MenuScreen object given a screen type and screen mode
     *
     */
    MenuScreen(ScreenType _screenType, ScreenMode _screenMode);
    
    MenuScreen();                           //default constructor
    MenuScreen(const MenuScreen & other);   //copy constructor
    
public:
    
    virtual ~MenuScreen();                  //deconstructor
    MenuScreen operator=(const MenuScreen & other); //= operator
    
    /**
     *  addButton(Button * button)
     *
     *  Adds the specified button to the menu and repaints.
     *
     */
    void addButton(Button * button);
    
    /**
     *  removeButton(Button * button);
     *
     *  Removes the specified button from the menu and repaints.
     *
     */
    void removeButton(Button * button);
    
    /**
     *  clearButtons();
     *
     *  Removes all buttons from the menu and repaints. (Warning: Also removes any other UIElements contained in the elementList of this object.)
     *
     */
    void clearButtons();
    
    /**
     *  findButton(char * name);
     *
     *  Returns a button with the specified name starting at index 0 in buttonList. Returns NULL if not found.
     *
     */
    Button * findButton(char * name);
    
    /**
     *  findButton(char * name, Button * button);
     *
     *  Returns a button with the specified naming, starting at the index of the specified button in buttonList. Returns NULL if not found.
     *
     */
    Button * findButton(char * name, Button * button);
    
    /**
     *  performAutoLayout()
     *
     *  Changes all the Button object positions and/or sizes in an organized manner.
     *
     */
    //void performAutoLayout();
    
    /**
     *  performAutoLayout(uint8 borderSpace)
     *
     *  Changes all the Button object positions and/or sizes in an organized manner with the specified border spacing (i.e. each button would be 10 px apart on all sides 'if (borderSpace == 10px)')
     *
     */
    //void performAutoLayout(uint8 borderSpace);
    
    /**
     *  paint()
     *
     *  Place code that paints the splash screen here.
     *
     *  Basic formula for splash and menu screens:
     *  paint backdrop, then paint contained elements (paintContainedElements())
     *
     */
    virtual void paint();
    
};

#endif

