/**
 *  SplashScreen.h
 *  
 *  A screen with an overridden paint method.
 *
 *  To use, create a child class and override the paint method with code for displaying the screen.
 *
 *  Created by Jaeder Patater on 11/7/05.
 *  Copyright 2005. All rights reserved.
 *
 */

#ifndef SPLASH_SCREEN_H
#define SPLASH_SCREEN_H

#include "Screen.h"

class SplashScreen : public Screen {
protected:
    
    //If constructors aren't given in a child class definition, does the compiler just use the parent class constructors? 
    //In this case, we want to define the constructors as protected so that the programmer -knows to/must- use this class as abstract. However, it would be useful to know in other applications.
    
    /**
     *  SplashScreen(ScreenType _screenType, ScreenMode _screenMode)
     *
     *  Creates a SpashScreen object given a screen type and screen mode.
     *
     */
    SplashScreen(ScreenType _screenType, ScreenMode _screenMode);
    
    SplashScreen();                             //default constructor
    SplashScreen(const SplashScreen & other);   //copy constructor
    
public:
    
    virtual ~SplashScreen();                    //deconstructor
    SplashScreen operator=(const SplashScreen & other); //= operator
    
    /**
     *  paint()
     *
     *  Place code that paints the splash screen here.
     *
     *  Basic formula for splash and menu screens:
     *  paint backdrop, then paint contained elements (paintContainedElements())
     *
     */
    virtual void paint();
    
};

#endif

