/**
 *  TouchEvent.h
 *  
 *
 *  Created by Jaeder Patater on 8/5/05.
 *  Copyright 2005. All rights reserved.
 *
 */

#ifndef TOUCH_EVENT_H
#define TOUCH_EVENT_H

#include "UIEvent.h"

class TouchEvent : public UIEvent {
private:
    
    /**
     *  init(const TouchEvent & other)
     *
     *  initializes this to other
     *
     */
    virtual void init(const TouchEvent & other);
    
    /**
     *  x,y
     *
     *  Stores the position of the touch.
     *
     */
    uint16 x;
    uint16 y;
    
    
public:
    
    /**
     *  TouchEvent(UIEventStatus _status, Coordinate _position)
     *
     *  Main constructor. Creates a TouchEvent object based on a given type, status, and position.
     *
     */
    TouchEvent(UIEventStatus status, uint16 x, uint16 y);
    
    TouchEvent();                                   //default constructor
    TouchEvent(const TouchEvent & other);           //copy constructor
    virtual ~TouchEvent();                          //deconstructor
    TouchEvent operator=(const TouchEvent & other); //= operator
    
    /**
     *  getX()
     *
     *  Returns the x coordinate where the touch occured
     *
     */
    uint16 getX() const;
    
    /**
     *  getY()
     *
     *  Returns the y coordinate where the touch occured
     *
     */
    uint16 getY() const;
    
};

#endif

