/**
 *  UIElement.h
 *  
 *  Defines a basic element in a user interface. Could be a button, a background, a sprite, etc.
 *  UIElements cannot recieve focus by default.
 *  In the Guikit model, all UIElement objects are UIEventManager objects. The reasoning behind this is so that 
 *  implementing windows and tabs and other containers of elements will be quite simple.
 *
 *  Created by Jaeder Patater on 7/28/05.
 *  Copyright 2005 PataterSoft. All rights reserved.
 *
 */

#ifndef UIELEMENT_H
#define UIELEMENT_H

//stdio (for NULL keyword among other things)
#include <stdio.h>


#include "KeyEvent.h"
#include "TouchEvent.h"
#include "UIEventManager.h"

//Handlers for events
//I'm not yet sure how to use these with the EventHandlerInterface as of yet. If you figure out the puzzle, let me know.
//typedef void (* EventHandler)(UIEvent * event);
//typedef void (* TouchHandler)(TouchEvent * touchEvent);
//typedef void (* KeyHandler)(KeyEvent * keyEvent);

//I have solved the puzzle.
/**
 *  7 November 2005 Update
 *  I have decided to ditch this solution and function pointers all together. This header file attempted to help create polymorphic
 *  function pointers. I had not realized that fact until only this morning (sitting in Japan). However, at the suggestion of Vince
 *  (DarkainMX), I have decided to go 100% object oriented and the more efficient V-tables and already accomplished standard method
 *  of polymorphism. Guikit will now be soley classes and methods, with no more function pointers. I'm pleased with myself that I had
 *  discovered the problem and connected it to polymorphism. I had been trying to do polymorphism without realizing it. Now the solution
 *  will be so much easier. I hope that the guikit is much improved by this change.
 */

/**
 *  11 November 2005 Update
 *  This method has turned out to be awesome. It has worked very well for me, even just 4 days after, I have paint() and setFocus() all 
 *  working great. I fell very happy now. Woo.
 *
 *  Patater in Japan
 *
 */


class UIElement : public UIEventManager {
protected:
    UIEventManager * parentContainer; //pointer to the parent container of this element (allows uielements contained by another to draw themselves). An element may only have one parent container, unless you do some crazy hacks or whatnot. Maybe later I should force the removal of an element from another container when you add it to a container (in UIEventManager add method, getParentContainer, if not NULL, remove element from that container and then add as usual)
    
    bool focusable;
    
    /**
     some UIEventManagers may not have any commands in them to paint to the screen, however. this creates a bit of a problem. Perhaps I should introduce a 
     new class, DrawableContainer, and require some draw functions and have DrawableContainer inherit from UIEventManager. Until then, just be careful when casting the parent container as a Screen, Window, or Tab (or any other drawable container).
    */
    
    UIElement();
    UIElement(const UIElement & other);         //copy constructor
    UIElement(UIElementList * _elementList);
    
    //perhaps put these into their own class, like UIElementListNode or some such, and inherit from it
    UIElement * previousElement;
    UIElement * nextElement;
    
public:
    
    virtual ~UIElement(); //destructor
    UIElement operator=(const UIElement & other);   //= operator
    
    /**
     *  getParentContainer()
     *
     *  Returns parentContainer
     *
     */
    UIEventManager * getParentContainer();
    
    /** 
     *  setParentContainer(UIEventManager * _parentContainer)
     *
     *  Sets the parent container to the specified container. (Used by UIEventManager (for adding, UIElementList for removing) to automatically set parentContainers)
     *
     *  !!!Warning: Do not use this manually to move an element from container to container. Use the container's add method instead. The parentContainer will be set automatically by the UIEventManager (for adding, UIElementList for removing).
     *
     */
    void setParentContainer(UIEventManager * _parentContainer);
    
    //--UIElementListNode type of stuff--
    //perhaps put these into their own class, like UIElementListNode or some such, and inherit from it
    UIElement * getPrevious();
    UIElement * getNext();
    void setPreviousElement(UIElement * prev);
    void setNextElement(UIElement * next);
    //--End UIElementListNode type of stuff
    
    /**
     *  canRecieveFocus()
     *
     *  Returns whether or not this UIElement can recieve focus.
     *
     */
    bool canRecieveFocus();
    
    /**
     *  inFocus()
     *
     *  Returns true if the UIElement is in the focus of its parent container.
     *
     */
    bool inFocus(); //"Is your mother watching you?"
    
    /**
     *  handleTouch()
     *
     *  The function to be called when handling a touch event
     *
     */
    virtual void handleTouch(TouchEvent * touchEvent);
    
    /**
     *  handleKeys()
     *
     *  The function to be called when handling a key event
     *
     */
    virtual void handleKeys(KeyEvent * keyEvent);
    
    /**
     *  paint()
     *
     *  The model for painting has not yet been finalized. Please offer suggestions. For now, you must call paint() manually (method 1). Perhaps I should build the model in J2ME style.
     *  
     *  Method 1: Default behavior of any UIElement is to simply paint its contained elements. Paint is called whenever the programmer deems fit (no automatic painting).
     *            When paint is called on an object, all objects it contains should also have paint called (this is the default behavior). In this way, paint is not an event,
     *            but simply a process that when invoked, travels down the chain of command.
     *
     *  Method 2: The function to be called when handling a paint event (if we make paint into an event).
     *
     */
    virtual void paint();
    
};

#endif

