/**
 *  UIElementList.h
 *  
 *
 *  Created by Jaeder Patater on 8/9/05.
 *  Copyright 2005 . All rights reserved.
 *
 */

//a linked list of UIElement pointers with add and remove functions
//stores the size
//items with a higher index are displayed higher
//-item 0 in the array is on the bottom
//-item 256 is much closer to the top
//-if item 0 and 256 overlap (for touch rect (hotspot)), then item 256 is chosen
//--this means that when searching for touch rect (hotspot) matches, we should start from the end and go to the beginning (count down search)
//this list is not responsible for any memory management, you give it pointers, and it works with those, then you must delete memory you used to allocate them, manually

#ifndef UI_ELEMENT_LIST_H
#define UI_ELEMENT_LIST_H

#include <stdio.h>

#include "UIElement.h"
#include "LinkedList.h"


class UIElementList {
private:
    
    LinkedList<UIElement *> * list;
    
    /**
     *  init(const UIElementList & other)
     *
     *  Initializes this to other
     *
     */
    void init(const UIElementList & other);
    
    /**
     *  free()
     *
     *  Deletes the linked list
     *
     */
    void free();
    
    /**
     *  validIndex(int i)
     *
     *  returns true if the specified index is within range
     *
     */
    bool validIndex(int i) const;
    
public:

    UIElementList();                                        //default constructor
    UIElementList(const UIElementList & other);             //copy constructor
    ~UIElementList();                                       //deconstructor
    UIElementList operator=(const UIElementList & other);   //= operator
    
    /**
     *  get(int i)
     *
     *  Returns index i of the list, NULL if invalid index
     *
     */
    UIElement * get(int i) const;
    
    /**
     *  getList()
     *
     *  Returns the list as read-only.
     *
     */
    const LinkedList<UIElement *> * getList();
    
    /**
     *  add(UIElement * element)
     *
     *  Adds a UIElement to the screen
     *
     */
    void add(UIElement * element);
    
    /**
     *  set(int i, UIElement * element)
     *
     *  Sets element to be the element in the specified list location, returning what was previously there
     *
     */
    UIElement * set(int i, UIElement * element);
    
    /**
     *  insertAfter(UIElement * index, UIElement * element)
     *
     *  Inserts the specified element after the index element. If the index is not found, the element is not inserted.
     *
     */
    void insertAfter(UIElement * index, UIElement * element);
    
    /**
     *  insertBefore(UIElement * index, UIElement * element)
     *
     *  Inserts the specified element after the index element. If the index is not found, the element is not inserted.
     *
     */
    void insertBefore(UIElement * index, UIElement * element);
    
    /**
     *  remove(UIElement * element)
     *
     *  Removes an element from the list
     *
     */
    void remove(UIElement * element);
    
    /**
     *  remove(int i)
     *
     *  Removes an element from the list and returns it
     *
     */
    UIElement * remove(int i);
    
    /**
     *  clear()
     *
     *  Clears the list
     *
     */
    void clear();
    
    /**
     *  find(UIElement * element)
     *
     *  Return the index of the specified element, -1 for not found
     *
     */
    int find(UIElement * element);
    
    /**
     *  getSize()
     *
     *  Returns the size of this list
     *
     */
    int getSize() const;
    
    /**
     *  isEmpty()
     *
     *  Returns true if the list is empty, false if otherwise.
     *
     */
    bool isEmpty() const;
    
};

#endif

