/**
 *  UIEvent.h
 *  
 *  Defines a basic event in a user interface. Could be a touch, a key press, drag and drop, etc. as defined in UIEventType.h
 *
 *  Created by Jaeder Patater on 7/28/05.
 *  Copyright 2005 PataterSoft. All rights reserved.
 *
 */

#ifndef UI_EVENT_H
#define UI_EVENT_H

//libnds
#include "nds.h"

#include "UIEventType.h"
#include "UIEventStatus.h"


class UIEvent {
protected:
    
    /**
     *  init(const UIEvent & other)
     *
     *  initializes this to other
     *
     */
    virtual void init(const UIEvent & other);
    
    /**
     *  type
     *
     *  Stores the type of event this event is.
     *
     */
    UIEventType type;
    
    /**
     *  status
     *
     *  Stores the status of the event that occured.
     *
     */
    UIEventStatus status;
    
public:
    
    UIEvent();                                          //default constructor, not for real use, just so copying is easier when passing (among other reasons)
    UIEvent(const UIEvent & other);                     //copy constructor
    void operator=(const UIEvent & other);              //= operator
    bool operator==(const UIEvent & other) const;       //== operator
    virtual ~UIEvent();                                 //deconstructor 
    
    /**
     *  getType()
     *
     *  Returns the type of event this event is.
     *
     */
    virtual UIEventType getType() const;
    
    /**
     *  getStatus()
     *
     *  Returns the type of event this event is.
     *
     */
    virtual UIEventStatus getStatus() const;
    
};

#endif

