/**
 *  Button.cpp
 *  
 *
 *  Created by Jaeder Patater on 11/6/05.
 *  Copyright 2005. All rights reserved.
 *
 */

#include "Button.h"

//strcpy and such
#include <string.h>

void Button::init() {
    pressed = false;
    name = NULL;
}

void Button::init(char * _name) {
    pressed = false;
    name = _name;
}

void Button::init(const Button & other) {
    pressed = other.pressed;
    strcpy(name, other.name);
}

Button::Button(char * _name, uint16 x, uint16 y, uint16 width, uint16 height) : HotSpot(x, y, width, height) {
    init(_name);
}

Button::Button() : HotSpot() {
    init();
}

Button::Button(const Button & other) : HotSpot(other) {
    init(other);
}

Button::~Button() {
    //nothing
}


Button Button::operator=(const Button & other) {
    HotSpot::operator=(other);
    init(other);
    
    return *this;
}

void Button::press() {
    pressed = true;
    
    paint();    //before calling handleTouch(), so that it is drawn released without delay 
                //(if you want the button to be drawn released after the process is complete, just move paint to after handleTouch())
    
    handleTouchDown();
}

void Button::release() {
    pressed = false;
    
    paint();    //before calling handleTouch(), so that it is drawn released without delay 
                //(if you want the button to be drawn released after the process is complete, just move paint to after handleTouch())
    
    handleTouchUp();
}

char * Button::getName() {
    return name;
}

void Button::paint() {
    if (pressed) {
        if (inFocus()) {
            paintPressedFocused();
        } else {
            paintPressed();
        }
    } else {
        if (inFocus()) {
            paintReleasedFocused();
        } else {
            paintReleased();
        }
    }
}

void Button::recieveEvent(UIEvent * event) {
    if (event->getType() == TYPE_TOUCH) {
        //press
        //if touch is held or down
        if (event->getStatus() == STATUS_DOWN) {
            if (!pressed) { //don't constantly call paint (via press) when you are simply holding the button (not rapidly pressing it) every touch held event
                press();
            }
        }
        
        if (event->getStatus() == STATUS_HELD) {
            if (!pressed) { //first time entering the button bounds
                press();
            } else {
                //they are holding the button down
                handleTouchHeld();
            }
        }
        
        //release
        //if touch up
        if (event->getStatus() == STATUS_UP) {
            release();
        }
        
        if (event->getStatus() == STATUS_EXIT) {
            //soft release (release, but don't execute button code)
            pressed = false;
            paint();
        }
    }
    
    //get rid of the event, because nobody needs it anymore
    delete event;
}

/**
 *  Button::handleTouchUp()
 *
 *  Put the code you want to execute on button release into this method
 *
 */
void Button::handleTouchUp() {
    //nothing
}

/**
 *  handleTouchDown()
 *
 *  Put the code you want to execute on button press into this method
 *
 */
void Button::handleTouchDown() {
    //nothing
}

/**
 *  handleTouchHeld()
 *
 *  Put the code you want to execute on button hold into this method
 *
 */
void Button::handleTouchHeld() {
    //nothing
}

/**
 *  Button::paintPressed()
 *
 *  Paints in the pressed state.
 *  
 *  Put your paint code here.
 *
 */
void Button::paintPressed() {
    //nothing, and I'm not sure why the linker insists on having these blank functions, probably so that the vtables don't have NULL pointers I'd guess
    /*
     obj/Button.o:(.gnu.linkonce.r._ZTV6Button[vtable for Button]+0x3c): undefined reference to `Button::handleTouch()'
     obj/Button.o:(.gnu.linkonce.r._ZTV6Button[vtable for Button]+0x40): undefined reference to `Button::paintPressed()'
     obj/Button.o:(.gnu.linkonce.r._ZTV6Button[vtable for Button]+0x44): undefined reference to `Button::paintReleased()'
     */
}

/**
 *  Button::paintReleased()
 *
 *  Paints the object in the released state.
 *
 *  Put your paint code here.
 *
 */
void Button::paintReleased() {
    //nothing
}

/**
 *  paintFocusedPressed()
 *
 *  Paints the object in the focused and pressed state.
 *
 *  Put your paint code here.
 *
 */
void Button::paintPressedFocused() {
    //default simply paint pressed
    paintPressed();
}

/**
 *  paintFocusedReleased()
 *
 *  Paints the object in the focused and released state.
 *
 *  Put your paint code here.
 *
 */
void Button::paintReleasedFocused() {
    //default simply paint released
    paintReleased();
}

