/**
 *  Draggable.cpp
 *  
 *
 *  Created by Jaeder Patater on 11/25/05.
 *  Copyright 2005. All rights reserved.
 *
 */

#include "Draggable.h"

void Draggable::recordGrip(TouchEvent * event) {
    gripX = event->getX();
    gripY = event->getY();
}

void Draggable::recordLoc() {
    //record location
    prevX = x;
    prevY = y;
}

Draggable::Draggable(char * name, uint16 x, uint16 y, uint16 width, uint16 height) : Button(name, x, y, width, height) {
    prevX = x;
    prevY = y;
    gripX = 0;
    gripY = 0;
}

Draggable::Draggable() : Button() {
    prevX = 0;
    prevY = 0;
    gripX = 0;
    gripY = 0;
}

Draggable::Draggable(const Draggable & other): Button(other) {
    prevY = other.prevY;
    prevX = other.prevX;
    gripX = other.gripX;
    gripY = other.gripY;
}

Draggable::~Draggable() {
    //nothing
}

Draggable Draggable::operator=(const Draggable & other) {
    Button::operator=(other);
    
    prevY = other.prevY;
    prevX = other.prevX;
    gripX = other.gripX;
    gripY = other.gripY;
    
    return *this;
}

void Draggable::recieveEvent(UIEvent * event) {
    if (event->getType() == TYPE_TOUCH) {
        //press
        //if touch is held or down
        if (event->getStatus() == STATUS_DOWN) {
            if (!pressed) { //don't constantly call paint (via press) when you are simply holding the button (not rapidly pressing it) every touch held event
                
                //since this is first time entering button, record the entry point to use when repositioning the button
                recordGrip((TouchEvent *)event);
                
                press();
            }
        }
        
        if (event->getStatus() == STATUS_HELD) {
            if (!pressed) { //first time entering the button bounds
                
                //since this is first time entering button, record the entry point as the grip to use when repositioning the button
                recordGrip((TouchEvent *)event);
                
                press();
            } else {
                //they are holding the button down
                TouchEvent * touchEvent = (TouchEvent *)event;
                
                //1st - reposition, but don't allow dragging off the screen
                int newX = x + touchEvent->getX() - gripX;
                int newY = y + touchEvent->getY() - gripY;
                
                //Don't record grip on special case. This prevents repositioning the grip when smashing up against a wall or corner.
                //bool shouldRecordGrip = true;
                
                if (newX < 0) {
                    x = 0;
                    //shouldRecordGrip = false;
                } else if (newX + width > 256) {
                    x = 256 - width;
                    //shouldRecordGrip = false;
                } else {
                    x = newX;
                }
                
                if (newY < 0) {
                    y = 0;
                    //shouldRecordGrip = false;
                } else if (newY + height > 192) {
                    y = 192 - height;
                    //shouldRecordGrip = false;
                } else {
                    y = newY;
                }
                
                //x += touchEvent->getX() - gripX;
                //y += touchEvent->getY() - gripY;
                
                //2nd - update grip
                //if (shouldRecordGrip) {
                    recordGrip(touchEvent);
                //}
                
                //3rd - paint() before calling handleTouchHeld(), so that it is drawn released without delay 
                //(if you want the button to be drawn released after the process is complete, just move paint to after handleTouch())
                paint();
                
                //look through parent's element list from the index of this element down to 0, if any elements found within bounds of this draggable, call paint on them
                //should be just one method on UIEventManager
                //element is usually (this), a pointer to the element wanting to have elements under it painted
                //paintUnderElements(UIElement * element);
                //also give a position (optional, used when giving prevX and prevY prevWidth, prevHeight)
                //paintUnderElements(UIElement * element, int x, int y, int width, int height);
                //parentContainer->paintUnderElements(this, prevX, prevY, width, height);
                
                //record location (current location will become previous location after next reposition, which is useful for painting)
                recordLoc(); //location last painted
                
                //4th - call held handler
                handleTouchHeld();
            }
        }
        
        //release
        //if touch up
        if (event->getStatus() == STATUS_UP) {
            release();
        }
        
        if (event->getStatus() == STATUS_EXIT) {
            //soft release (release, but don't execute button code)
            pressed = false;
            paint();
        }
    }
    
    //get rid of the event, because nobody needs it anymore
    delete event;
}

void Draggable::handleTouchUp() {
    
}

void Draggable::handleTouchDown() {
    
}

void Draggable::handleTouchHeld() {
    
}

void Draggable::paintPressed() {
    
}

void Draggable::paintReleased() {
    
}

void Draggable::paintPressedFocused() {
    
}

void Draggable::paintReleasedFocused() {
    
}

