/**
 *  UIElement.cpp
 *
 *  Created by Jaeder Patater on 7/28/05.
 *  Copyright 2005. All rights reserved.
 *
 */

#include "UIElement.h"

UIElement::UIElement() : UIEventManager() {
    parentContainer = NULL;
    
    //list related
    previousElement = NULL;
    nextElement = NULL;
    
    focusable = false;
}

UIElement::UIElement(const UIElement & other) : UIEventManager(other) {
    parentContainer = other.parentContainer;
    
    //list related
    previousElement = other.previousElement;
    nextElement = other.previousElement;
    
    focusable = other.focusable;
}

UIElement::UIElement(UIElementList * _elementList) : UIEventManager(_elementList) {
    parentContainer = NULL;
    
    //list related
    previousElement = NULL;
    nextElement = NULL;
    
    focusable = false;
}

UIElement::~UIElement() {
    //nothing
}

UIElement UIElement::operator=(const UIElement & other) {
    UIEventManager::operator=(other);
    
    parentContainer = other.parentContainer;
    
    //list related
    previousElement = other.previousElement;
    nextElement = other.previousElement;
    
    focusable = other.focusable;
    
    return *this;
}

UIEventManager * UIElement::getParentContainer() {
    return parentContainer;
}

void UIElement::setParentContainer(UIEventManager * _parentContainer) {
    parentContainer = _parentContainer;
}

UIElement * UIElement::getPrevious() {
    return previousElement;
}

UIElement * UIElement::getNext() {
    return nextElement;
}

void UIElement::setPreviousElement(UIElement * prev) {
    previousElement = prev;
}

void UIElement::setNextElement(UIElement * next) {
    nextElement = next;
}

bool UIElement::canRecieveFocus() {
    return focusable;
}

bool UIElement::inFocus() {
    return (parentContainer->getFocusElement() == this);
}

void UIElement::handleTouch(TouchEvent * touchEvent) {
    //blank
    
    //blank so that an element has some function to fall back on. May or may not be necessary
    //(depends on how the compiler handles this sort of thing. I personally don't know how or can't remember (if I ever knew), so
    //I made these blank functions)
    
    //November 10 2005 update, these blank functions are required by the linker
    //I got a bunch of linker errors because I didn't put blank functions in other places
    //check my japan journal for more information
}

void UIElement::handleKeys(KeyEvent * keyEvent) {
    //blank
    
    //see reason for handleTouch for reason
}

void UIElement::paint() {
    //Default behavior of any UIElement is to simply paint its contained elements.
    paintContainedElements();
}

