/**
 *  UIElementList.cpp
 *  
 *  Linked list implementation of a UIElementList.
 *
 *  Created by Jaeder Patater on 8/9/05.
 *  Copyright 2005. All rights reserved.
 *
 */

#include "UIElementList.h"
#include "nds/arm9/console.h"


void UIElementList::init(const UIElementList & other) {
    list = other.list;
}

void UIElementList::free() {
    if (list != NULL) {
        //run through the list and let all elements know they have been disowned
        LLNode<UIElement *> * node = list->GetFirst();
        while (node != NULL) {
            node->GetValue()->setParentContainer(NULL);
            node = node->GetNext();
        }
        
        delete list;
        list = NULL;
    }
}

bool UIElementList::validIndex(int i) const {
    return ((i >=0) && (i < list->GetSize()));
}

UIElementList::UIElementList() {
    //create a new list
    list = new LinkedList<UIElement *>();
}

UIElementList::UIElementList(const UIElementList & other) {
    init(other);
}

UIElementList::~UIElementList() {
    free();
}

UIElementList UIElementList::operator=(const UIElementList & other) {
    free();
    
    init(other);
    
    return *this;
}

UIElement * UIElementList::get(int i) const {
    if (validIndex(i)) {
        //run through the list until you reach the i numbered element
        LLNode<UIElement *> * node = list->GetFirst();
        for (int j = 0; j < i; j++) {
            if (node != NULL) {
                node = node->GetNext();
            } else {
                //bad index? (should have been caught in the check)
                return NULL;
            }
        }
        
        return node->GetValue();
    } else {
        return NULL;
    }
}

const LinkedList<UIElement *> * UIElementList::getList() {
    return list;
}

void UIElementList::add(UIElement * element) {
    list->Insert(element, list->GetLast()); //add to end of list
}

UIElement * UIElementList::set(int i, UIElement * element) {
    if (validIndex(i)) {
        //run through the list until you reach the i numbered element
        LLNode<UIElement *> * node = list->GetFirst();
        for (int j = 0; j < i; j++) {
            if (node != NULL) {
                node = node->GetNext();
            } else {
                //bad index? (should have been caught in the check)
                return NULL;
            }
        }
        
        //store the value at index i temporarily
        UIElement * oldElement = node->GetValue();
        
        list->Insert(element, node);
        list->Remove(node);
        //element->setParentContainer(this); //nobody is doing the setting, and the list isn't the parent, the container is (when and if UIEventManager gets a set method, be sure to set parent container)
        
        //return the old value at index i
        return oldElement;
    } else {
        return NULL;
    }
}

void UIElementList::insertAfter(UIElement * index, UIElement * element) {
    LLNode<UIElement *> * node = list->Find(index, NULL);
    if (node != NULL) { //if the index is not found, the element is not inserted
        list->Insert(element, node);
    }
}

void UIElementList::insertBefore(UIElement * index, UIElement * element) {
    LLNode<UIElement *> * node = list->Find(index, NULL);
    if (node != NULL) { //if the index is not found, the element is not inserted
        list->Insert(element, node->GetPrevious());
    }
}

void UIElementList::remove(UIElement * element) {
    list->Remove(list->Find(element, NULL));
}

UIElement * UIElementList::remove(int i) {
    if (validIndex(i)) {
        //run through the list until you reach the i numbered element
        LLNode<UIElement *> * node = list->GetFirst();
        for (int j = 0; j < i; j++) {
            if (node != NULL) {
                node = node->GetNext();
            } else {
                //bad index? (should have been caught in the check)
                return NULL;
            }
        }
        
        //store the value at index i temporarily
        UIElement * oldElement = node->GetValue();
        
        list->Remove(node);
        
        //let the old element know that it has been disowned
        oldElement->setParentContainer(NULL);
        
        //return the old value at index i
        return oldElement;
    } else {
        return NULL;
    }
}

void UIElementList::clear() {
    list->Clear();
}

int UIElementList::find(UIElement * element) {
    //perform a linear search for the specified element
    LLNode<UIElement *> * node = list->GetFirst();
    int i = 0;
    while (node != NULL) {
        if (node->GetValue() == element) {
            return i;
        }
        node = node->GetNext();
        i++;
    }
    
    return -1; //returns -1 for not found
}

int UIElementList::getSize() const {
    return list->GetSize();
}

bool UIElementList::isEmpty() const {
    return list->IsEmpty();
}

